/****************************************************************************
 *
 * vla-demo.c - Variable-length arrays demo
 *
 * Last updated in 2017 by Moreno Marzolla <moreno.marzolla(at)unibo.it>
 *
 * To the extent possible under law, the author(s) have dedicated all 
 * copyright and related and neighboring rights to this software to the 
 * public domain worldwide. This software is distributed without any warranty.
 *
 * You should have received a copy of the CC0 Public Domain Dedication
 * along with this software. If not, see 
 * <http://creativecommons.org/publicdomain/zero/1.0/>. 
 *
 * --------------------------------------------------------------------------
 *
 * Compile with:
 *
 * gcc -std=c99 -Wall -Wpedantic vla-demo.c -o vla-demo
 *
 * Run with:
 *
 * ./vla-demo
 *
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>

void identity_mat( int *mat, size_t n )
{
    /* mat_m is a pointer to an array[n] of integers */
    int (*mat_m)[n] = (int (*)[n])mat;
    size_t i, j;
    for (i=0; i<n; i++) {
        for (j=0; j<n; j++) {
            mat_m[i][j] = (i == j ? 1 : 0); /* un po' ridondante, ma comprensibile */
        }
    }
}

void print_mat( const int *mat, size_t n )
{
    /* mat_m is a pointer to an array[n] of (constant) integers */
    const int (*mat_m)[n] = (const int (*)[n])mat;
    size_t i, j;
    for (i=0; i<n; i++) {
        for (j=0; j<n; j++) {
            printf("%d\t", mat_m[i][j]);
        }
        printf("\n");
    }
}

int main( void )
{
    const size_t n = 8;
    int *m = (int*)malloc(n*n*sizeof(int));
    identity_mat(m, n);
    print_mat(m, n);
    free(m);
    return EXIT_SUCCESS;
}
